@@ -8,6 +8,7 @@ import secrets
 from pathlib import Path
 from datetime import datetime
 from functools import lru_cache
+from io import StringIO
 import boto3
 from botocore.exceptions import NoCredentialsError, ClientError
 import numpy as np
@@ -208,6 +209,7 @@ in your notebook with: `tracker.CACHE_SIZE = `"""
             print(msg)
 
             bucket_name = base_dir[:base_dir.find("/", 1)]
+            self.bucket_name = bucket_name
             try:
                 s3 = boto3.resource('s3')
                 s3.meta.client.head_bucket(Bucket=bucket_name)
@@ -475,7 +477,7 @@ with a different name OR manually delete the existing stuff locally.
             run = secrets.token_hex(3)
 
         passed_attr = kwargs.keys()
-        this_run = self.get_run(run=run, verbose=verbose)
+        this_run = self.get_run(run=run)
 
         pops = []
         existing_references = {}
@@ -634,7 +636,6 @@ with a different name OR manually delete the existing stuff locally.
         """
         @doc: get the run's dictionary wihtout loading model and prep objects
         @args: run is a 6 digit string identifier for the run
-               verbose is a boolean for whether or not to print anything
         @return: a dict containing the run's information
         """
         remote = self.remote
@@ -691,7 +692,6 @@ with a different name OR manually delete the existing stuff locally.
         """
         @doc: load the reference lists for field names
         @args: run is a str of the run ID
-               verbose is a boolean flag for whether or not to print status
         @return: a dictionary of the names and lists of references
         """
         remote = self.remote
@@ -716,7 +716,7 @@ with a different name OR manually delete the existing stuff locally.
 
         return ret
 
-    def store_data(self, data, set_type="X_train", index=False):
+    def store_data(self, data, set_type="X_train", index=False, csv_buffer=None, chunk=None):
         """
         @doc: a convenience method for writing data to the local disk
         @args: data is either a text blob (probably representing some SQL) or
@@ -724,7 +724,7 @@ with a different name OR manually delete the existing stuff locally.
                set_type is one of the allowable set types (e.g. 'X_train'... see tracker.set_types)
                index is a boolean for whether or not to write the index to CSV
                    (only used if data is a pandas DataFrame)
-               verbose is a boolean for whether or not to print anything
+                csv_buffer is a StringIO buffer to use in writing the data if it's pandas and chunk is not None
         @return: a tuple of the run string and the data path string
         @note: if no run is specified it will initialize a new identifier...
         """
@@ -750,7 +750,13 @@ with a different name OR manually delete the existing stuff locally.
                     file.write(data)
                 bucket.upload_file(Key=data_str, Filename=filename)
                 os.remove(filename)
-
+            elif isinstance(csv_buffer, StringIO) and chunk is not None:
+                data_str = "{0}/data/{1}_{2}.csv".format(self.project_dir, set_type, run)
+                s3 = boto3.resource('s3')
+                bucket_name = self.bucket_name
+                data.to_csv(csv_buffer, index=index, header=True)
+                s3.Object(bucket_name, "{0}{1}.csv".format(data_str[:-4], chunk)).put(Body=csv_buffer.getvalue())
+                data_str = (data_str, chunk)
             else:
                 filename = "{0}/{1}/temp_data.csv".format(HM, name)
                 data_str = "{0}/data/{1}_{2}.csv".format(self.project_dir, set_type, run)
@@ -798,7 +804,7 @@ with a different name OR manually delete the existing stuff locally.
         _ = self.store_run(run=run, **kwarg)
         return run, data_str
 
-    def load_data(self, run=None, set_type="X_train", delim=",", skiprows=0):
+    def load_data(self, run=None, set_type="X_train", delim=",", skiprows=0, chunks=None):
         """
         @doc: get either an array of data OR a text blob (presumably a query of some kind)
         @args: run is a str of the 6 digit run identifier
@@ -825,6 +831,8 @@ with a different name OR manually delete the existing stuff locally.
 
         run_dict = self.get_run(run)
         data_str = run_dict[set_type]
+        if isinstance(data_str, list):
+            data_str = data_str[0]
 
         if remote:
             if data_str[-3:].lower() == 'txt':
@@ -840,7 +848,16 @@ with a different name OR manually delete the existing stuff locally.
                     if verbose:
                         print("loaded TXT file to a blob")
                 os.remove(filename)
-
+            elif data_str[-3:].lower() == 'csv' and chunks is not None:
+                filename = "{0}/{1}/temp_data.csv".format(HM, name)
+                for i, chunk in enumerate(chunks):
+                    chunk_str = "{0}{1}.csv".format(data_str[:-4], chunk)
+                    bucket.download_file(Key=chunk_str, Filename=filename)
+                    if i == 0:
+                        ret = pd.read_csv(filename, sep=delim)
+                    else:
+                        df = pd.read_csv(filename, sep=delim)
+                        ret = pd.concat([ret, df], ignore_index=True)
             elif data_str[-3:].lower() == 'csv':
                 filename = "{0}/{1}/temp_data.csv".format(HM, name)
 
@@ -904,6 +921,7 @@ Failing that, loading it as a text blob.""".format(delim)
         @note: this will only remove things that are named according to the run identifier.
                (i.e. it will not remove data referred to by the run if that data is based on
                 a different run)
+        @todo: find a better way to check for run data to remove
         """
         project_dir = self.project_dir
         run_dict = RUN_ATTRIBUTES.copy()
@@ -929,7 +947,15 @@ Failing that, loading it as a text blob.""".format(delim)
 
             for set_type in SET_TYPES:
                 filename = run_dict[set_type]
-                if len(filename) > 0 and s3_isfile(filename, bucket) and run in filename:
+                if isinstance(filename, list):
+                    basefile = filename[0]
+                    chunks = filename[1]
+                    for i in range(chunks):
+                        filename = "{0}{1}.csv".format(basefile[:-4], i)
+                        if s3_isfile(filename, bucket):
+                            bucket.Object(key=filename).delete()
+
+                elif len(filename) > 0 and s3_isfile(filename, bucket) and run in filename:
                     bucket.Object(key=filename).delete()
 
             for _, val in references.items():
